/* This file is part of xdir, an X-based directory browser.
 * It contains routines for dealing with actual directories.
 *
 *	Created: 8/13/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/dir.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_xdir_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/dir.c,v 3.0 90/03/31 01:35:18 treese Exp $";
#endif

#include <sys/types.h>
#include <sys/dir.h>
#include <X11/Intrinsic.h>

#include "xdir.h"

#define NENTRIES	20

/* Function:	SPComp() compares two string pointers for qsort().
 * Arguments:	s1, s2: strings to be compared.
 * Returns:	Value of strcmp().
 * Notes:
 */

static int
SPComp(s1, s2)
char **s1, **s2;
{
	return(strcmp(*s1, *s2));
}

char **
MakeFileList(dir_name)
char *dir_name;
{
	DIR *dirp;
	struct direct *dp;
	char **filelist;
	char **cur_file;
	char **last_file;
	int entry_cnt;

	entry_cnt = NENTRIES;
	filelist = (char **) calloc(entry_cnt, sizeof (char *));
	cur_file = filelist;
	last_file = filelist + entry_cnt - 1;

	dirp = opendir(dir_name);
	if (dirp == NULL) {
		return(NULL);
	}
	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
		if ( !strcmp(dp->d_name, ".") )
			continue;
		if ( (! resources.ShowDotFiles) && (dp->d_name[0] == '.'))
			continue;
		if ( (! resources.ShowBackupFiles) &&
		    (dp->d_name[dp->d_namlen-1] == TWIDDLE))
			continue;
		*cur_file++ = SaveString(dp->d_name);
		if (cur_file == last_file) {
			filelist = (char **) realloc(filelist,
					    2 * entry_cnt * sizeof (char *));
			cur_file = filelist + entry_cnt - 1;
			entry_cnt = 2 * entry_cnt;
			last_file = filelist + entry_cnt - 1;
		}
	}
	*cur_file = NULL;
	qsort(filelist, cur_file - filelist - 1, sizeof (char *), SPComp);
	return(filelist);
}

/* Function:	PrintFileList() prints a list of files from an array.
 * Arguments:	file_list:	List to print.
 * Returns:	Nothing.
 * Notes:
 */

void
PrintFileList(file_list)
char **file_list;
{
	if (file_list == NULL)
		return;
	while(*file_list != NULL)
		puts(*file_list++);
}
