/* This file is part of xdir, an X-based directory browser.
 * It contains functions for displaying a file.
 *
 *	Created: 8/13/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/display.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_display_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/display.c,v 3.0 90/03/31 01:35:19 treese Exp $";
#endif

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Dialog.h>

#include "xdir.h"

/* Local variables. */

static Widget TextShell;
static Widget TextBox;
static Widget Text;
static Widget TextPane;
static Boolean textPopped;
static Widget TextLabel;
static Boolean ForceClose = False;

/* Function:	GetFileName() gets the filename from the text widget.
 * Arguments:	Widget to retrieve name from.
 * Returns:	char * -- name of file.
 * Notes:
 */

char *
GetFileName(w)
Widget w;
{
	static char *old;
	static Arg getargs[] = { {XtNstring, (XtArgVal) &old} };

	XtGetValues(w, getargs, (Cardinal) 1);
	return(old);
}

/* Function:	TextHelp() displays help on the file display widget.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing
 * Notes:
 */

/*ARGSUSED*/
void
TextHelp(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	char helpfile[MAXSTRLEN];

	sprintf(helpfile, "%s/%s", resources.LibDir, resources.ViewHelpFile);
	ShowHelp(helpfile, "Xdir: Help on Viewing Files");
}

/* Function:	TextClose() closes a file display window.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
TextClose(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
#ifdef notdef
	if ( (XawAsciiSourceChanged(w) == True) && (ForceClose == False)) {
		SaveDialog(w);
	}
#endif
	XtPopdown(TextShell);
	textPopped = False;
}

#ifdef notdef
/* Function:	TextSave() saves a file edited in the display window.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
TextSave(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if (XawAsciiSave(XawTextGetSource(w)) != True) {
		Error("Unable to save file", GetFileName(w));
	}
}
#endif

/* Function:	InitTextDisplay() initializes the text display windows.
 * Arguments:	None.
 * Returns:	Nothing.
 * Notes:
 */

void
InitTextDisplay()
{
	Arg args[10];
	Cardinal arg_cnt;
	Widget w;

	TextShell = XtCreatePopupShell("TextShell",
					     topLevelShellWidgetClass,
					     TopWidget,
					     NULL, 0);

	arg_cnt = 0;
  	TextPane = XtCreateManagedWidget("text-pane", panedWidgetClass,
				     TextShell, args, arg_cnt);

	arg_cnt = 0;
	TextLabel = XtCreateManagedWidget("textname", labelWidgetClass, 
				      TextPane, args, arg_cnt);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNtype, XawAsciiFile); arg_cnt++;
	XtSetArg(args[arg_cnt], XtNeditType, XawtextRead); arg_cnt++;
/*	XtSetArg(args[arg_cnt], XtNresize, XawtextResizeHeight); arg_cnt++; */
	XtSetArg(args[arg_cnt], XtNstring, "/dev/null"); arg_cnt++;
	Text = XtCreateManagedWidget("ViewText", asciiTextWidgetClass,
				     TextPane,	args, arg_cnt);

	arg_cnt = 0;
	TextBox = XtCreateManagedWidget("text-box", boxWidgetClass, TextPane,
					args, arg_cnt);
	

	arg_cnt = 0;
        w = XtCreateManagedWidget("close", commandWidgetClass, TextBox,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, TextClose, NULL);

#ifdef notdef
	arg_cnt = 0;
        w = XtCreateManagedWidget("save", commandWidgetClass, TextBox,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, TextSave, NULL);
#endif

	arg_cnt = 0;
        w = XtCreateManagedWidget("help", commandWidgetClass, TextBox,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, TextHelp, NULL);

	XtRealizeWidget(TextShell);
}

/* Function:	DisplayFile() displays a text file.
 * Arguments:	filename:	Name of file to display.
 * Returns:	Nothing.
 * Notes:
 */

void
DisplayFile(filename, edit)
char *filename;
Boolean edit;
{
	Arg args[10];
	Cardinal arg_cnt;
	char fullpath[MAXPATHLEN];
	int access_mode;
	struct stat info;

	MakeFullPath(filename, fullpath);

	access_mode = R_OK | ((edit) ? W_OK : 0);
	if ( access(fullpath, access_mode) ) {
		Error("Cannot read file", fullpath);
		return;
	}
	if (stat(fullpath, &info) != 0) {
		Error("Cannot open file", fullpath);
		return;
	}
	switch (info.st_mode & S_IFMT) {
	case S_IFDIR:
		Error(fullpath, "is a directory.");
		return;
	case S_IFCHR:
		Error(fullpath, "is a character special file.");
		return;
	case S_IFBLK:
		Error(fullpath, "is a block special file.");
		return;
#ifdef S_IFSOCK
	case S_IFSOCK:
		Error(fullpath, "is a socket.");
		return;
#endif /*S_IFSOCK*/
#ifdef S_IFIFO
	case S_IFIFO:
		Error(fullpath, "is a named pipe (FIFO).");
		return;
#endif /*S_IFIFO*/
#ifdef S_IFLNK:
	case S_IFLNK:
#endif /*S_IFLNK*/
	case S_IFREG:
		break;
	default:
		Error("Unknown file type:", fullpath);
		return;
	}

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, fullpath); arg_cnt++;
	XtSetValues(TextLabel, args, arg_cnt);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNeditType,
		 ((edit) ? XawtextEdit : XawtextRead)); arg_cnt++;
	XtSetValues(Text, args, arg_cnt);

#ifdef notdef
	XtSetSensitive(XtNameToWidget(TextBox, "save"), edit);
#endif
	SetTextFile(Text, fullpath);

	if (! textPopped) {
		XtPopup(TextShell, XtGrabNone);
		textPopped = True;
	}
}

/* Function:    SetTextFile() sets the text file of the specified widget.
 * Arguments:   w:              Widget to set.
 *              filename:       Name of filename.
 * Returns:     Nothing.
 * Notes:
 * XXX The bookkeeping involved here is necessary because the Text widget
 * uses pointer comparison to detect if the string has actually changed.
 */

void
SetTextFile(w, filename)
Widget w;
char *filename;
{
        Arg args[10];
        Cardinal arg_cnt;
	char *new;
	char *old;

	old = GetFileName(w);

	/* Make a copy of the new name and modify the widget. */
       
	new = SaveString(filename);
        arg_cnt = 0;
        XtSetArg(args[arg_cnt], XtNstring, new); arg_cnt++;
        XtSetValues(w, args, arg_cnt);

	/* ...and free the old name, now that it's no longer used. */
	free(old);
}

#ifdef notdef

static Widget SaveShell;

/* Function:	EndSaveDialog() removes the SaveDialog box.
 * Arguments:	None.
 * Returns:	Nothing.
 * Notes:
 */

void
EndSaveDialog()
{
	XtPopdown(SaveShell);
	XtDestroyWidget(SaveShell);
}

/* Function:	SaveClose() saves the file and closes the file view.
 * Arguments:	Standard Xt callbacks.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
SaveClose(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	TextSave(w, NULL, NULL);	/* Simulate callback to save text. */
	EndSaveDialog();
	TextClose(w, NULL, NULL);	/* Simulate callback to close view. */
}

/* Function:	JustClose() closes the file view without saving the file.
 * Arguments:	Standard Xt callbacks.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
JustClose(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	EndSaveDialog();
	ForceClose = True;
	TextClose();
	ForceClose = False;
}

/* Function:	CancelSave() cancels the save and returns to the file view.
 * Arguments:	Standard Xt callbacks.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
CancelSave(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	EndSaveDialog();
}

/* Function:	SaveDialog() activates when the user closes a modified
 *		        buffer.
 * Arguments:	w:	Modified widget.
 * Returns:	Nothing.
 * Notes:
 */

void
SaveDialog(w)
Widget w;
{
	char message[BUFSIZ];
	Arg args[10];
	Cardinal arg_cnt;
	Widget w, dialog;
	int x, y;
	Position x_root, y_root;
	Widget SaveShell;

	sprintf(message, "File %s has not been saved.", GetFileName(w));

	SaveShell = XtCreatePopupShell("SaveShell",
					     transientShellWidgetClass,
					     TextShell,
					     NULL, 0);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, message); arg_cnt++;
	dialog = XtCreateManagedWidget("SaveDialog", dialogWidgetClass,
					    SaveShell, args, arg_cnt);

	arg_cnt = 0;
        w = XtCreateManagedWidget("save", commandWidgetClass, dialog,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, SaveClose, NULL);

	arg_cnt = 0;
        w = XtCreateManagedWidget("close", commandWidgetClass, dialog,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, JustClose, NULL);

	arg_cnt = 0;
        w = XtCreateManagedWidget("close", commandWidgetClass, dialog,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, CancelSave, NULL);

	XtRealizeWidget(SaveShell);

	/* Center the dialog box in the directory widget. */

	x = (GetWidgetWidth(TextShell) - GetWidgetWidth(SaveShell)) / 2;
	y = (GetWidgetHeight(TextShell) - GetWidgetHeight(SaveShell)) / 2;
	if (x < 0) x = 0;
	if (y < 0) y = 0;

	XtTranslateCoords(TextShell, x, y, &x_root, &y_root);

	XtSetArg(args[arg_cnt], XtNx, x_root); arg_cnt++;
	XtSetArg(args[arg_cnt], XtNy, y_root); arg_cnt++;
	XtSetValues(SaveShell, args, arg_cnt);

	XtPopup(SaveShell, XtGrabNonexclusive);
}
#endif
