/* This file is part of xdir, an X-based directory browser.
 * It contains functions for the "File" menu.
 *
 *	Created: 10/8/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/filemenu.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_xdir_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/filemenu.c,v 3.0 90/03/31 01:35:21 treese Exp $";
#endif

#include <stdio.h>
#include <sys/param.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/List.h>

#include "xdir.h"

/*ARGSUSED*/
static void
Update(w, client_data, ret_val)
Widget w;
caddr_t client_data;
caddr_t ret_val;
{
	DoChangeDir(NULL);

}

/* Function:	Quit() exits the program.  It is called from the "QUIT" button.
 * Arguments:	Standard Xt callback.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
static void
Quit(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	exit(0);
}

/* Function:	ViewFile() displays a selected file.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
static void
ViewFile(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if (CurrentSelectionName[0] != EOS)
		DisplayFile(CurrentSelectionName, NOEDIT);
	else
		Error("You have not selected a file to view.", NULL);
}

/*ARGSUSED*/
static void
AboutXdir(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	char helpfile[MAXSTRLEN];

	sprintf(helpfile, "%s/%s", resources.LibDir, resources.AboutFile);
	ShowHelp(helpfile, "About Xdir");
}

/* Function:	ChangeDir() changes to the selected directory.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
static void
ChangeDir(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	if (CurrentSelectionName[0] == EOS) {
		Error("No directory selected.", "");
		return;
	}
	if (! IsDirectory(CurrentSelectionName)) {
		Error(CurrentSelectionName, "is not a directory.");
		return;
	}
	DoChangeDir(CurrentSelectionName);
}

/* Function:	ParentDir() changes to the parent directory.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
static void
ParentDir(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	DoChangeDir("..");
}

/* Function:	DoChangeDir() actually changes the directory and displays
 *		the new listing.
 * Arguments:	dir:	Pathname of new directory.
 * Returns:	Nothing.
 * Notes:
 *	NULL for dir means to rebuild the file list for the current directory
 *	(as in an update to the directory or change in filename filter).
 */

void
DoChangeDir(dir)
char *dir;
{
	char **filelist;
	char *p;
	Arg args[10];
	Cardinal arg_cnt;
	char newdir[MAXPATHLEN];

	strcpy(newdir, CurrentDirectory);
	if (dir != NULL) {
		if ( !strcmp(dir, "..")) {	/* Parent directory. */
			p = rindex(newdir, '/');
			*p = EOS;
			if (newdir[0] == EOS)
			  strcpy(newdir, "/");
		}
		else	{
			if ( strcmp(newdir, "/") ) /* At the root already */
			  strcat(newdir, "/");
			strcat(newdir, dir);
		}
	}
	if ( (filelist = MakeFileList(newdir)) == NULL) {
		Error("Unable to list directory", newdir);
		return;
	}

	strcpy(CurrentDirectory, newdir);
	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, CurrentDirectory); arg_cnt++;
	XtSetValues(DirLabel, args, arg_cnt);

	SetDirIconName(CurrentDirectory);
	XawListChange(MyListWidget, filelist, 0, 0, True);
	CurrentSelectionName[0] = '\0';
}

/*ARGSUSED*/
static void
ItemPicked(w, event, params, param_count)
Widget w;
XEvent *event;
String *params;
Cardinal *param_count;
{
	if (IsDirectory(CurrentSelectionName))
	  DoChangeDir(CurrentSelectionName);
	else
	  DisplayFile(CurrentSelectionName, NOEDIT);
}


/* The following static tables support the translation table changes.
 * They are down here in order to take advantage of the functions declarations
 * above.
 */

static XtActionsRec actionTable[] = {
  {"ParentDir", ParentDir},
  {"ItemPicked", ItemPicked},
};

/* Function:    InitButtonActions() initializes other mouse actions.
 * Arguments:   None.
 * Returns:     Nothing.
 * Notes:
 */

void
InitButtonActions()
{
        XtAppAddActions(XdirContext, actionTable, XtNumber(actionTable));
}

/* Function:	InitFileMenu() initializes the "Actions" menu.
 * Arguments:	parent:	Parent widget to use.
 * Returns:	Nothing.
 * Notes:
 */

void
InitFileMenu(parent)
Widget parent;
{
	Widget menu;

	(void) XtCreateManagedWidget("file", menuButtonWidgetClass, parent,
					NULL, (Cardinal) 0);

	menu = XtCreatePopupShell("fileMenu", simpleMenuWidgetClass,
				  TopWidget, NULL, (Cardinal) 0);

	(void) AddMenuEntry(menu, "view", ViewFile);
	(void) AddMenuEntry(menu, "chdir", ChangeDir);
	(void) AddMenuEntry(menu, "parentdir", ParentDir);
	(void) AddMenuEntry(menu, "about", AboutXdir);
	(void) AddMenuEntry(menu, "update", Update);
	(void) AddMenuEntry(menu, "quit", Quit);
}
