/* This file is part of xdir, an X-based directory browser.
 * It contains functions for displaying help.
 *
 *	Created: 9/5/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/help.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_help_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/help.c,v 3.0 90/03/31 01:35:23 treese Exp $";
#endif

#include <stdio.h>
#include <sys/param.h>
#include <X11/Xos.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Text.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>

#include "xdir.h"

/* Local variables. */

static Widget HelpShell = NULL;

/*ARGSUSED*/
static void
Help(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	char helpfile[MAXSTRLEN];

	sprintf(helpfile, "%s/%s", resources.LibDir, resources.HelpFile);
	ShowHelp(helpfile, "Xdir Help");
}

/* Function:	HelpClose() closes a help window.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 */

/*ARGSUSED*/
void
HelpClose(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
	XtPopdown(HelpShell);
}

/* Function:	CreateHelpWidget() creates the help widget.
 * Arguments:	None.
 * Returns:	Nothing.
 * Notes:
 */

static void
CreateHelpWidget()
{
	Arg args[10];
	Cardinal arg_cnt;
	Widget pane, box, close;

	HelpShell = XtCreatePopupShell("help", topLevelShellWidgetClass,
					     TopWidget,
					     NULL, 0);

  	pane = XtCreateManagedWidget("pane", panedWidgetClass, HelpShell,
				     NULL, (Cardinal) 0);
	(void) XtCreateManagedWidget("helplabel", labelWidgetClass, pane,
				     NULL, (Cardinal) 0);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNtype, XawAsciiFile); arg_cnt++;
	XtSetArg(args[arg_cnt], XtNeditType, XawtextRead); arg_cnt++;
	XtSetArg(args[arg_cnt], XtNstring, "/dev/null"); arg_cnt++;
	(void) XtCreateManagedWidget("text", asciiTextWidgetClass, pane,
				     args, arg_cnt);

	box = XtCreateManagedWidget("box", boxWidgetClass, pane,
				     NULL, (Cardinal) 0);
        close = XtCreateManagedWidget("close", commandWidgetClass, box,
				     NULL, (Cardinal) 0);
	XtAddCallback(close, XtNcallback, HelpClose, NULL);

	XtRealizeWidget(HelpShell);
}

/* Function:	ShowHelp() displays a help window.
 * Arguments:	helpfile:	File to display in the window.
 *		label:		Label for window.
 * Returns:	Nothing.
 * Notes:
 */

void
ShowHelp(helpfile, label)
char *helpfile;
char *label;
{
	Arg args[10];
	Cardinal arg_cnt;

	if (HelpShell == NULL)
		CreateHelpWidget();


	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, label); arg_cnt++;
	XtSetValues(XtNameToWidget(HelpShell, "pane.helplabel"),
		    args, arg_cnt);

	SetTextFile(XtNameToWidget(HelpShell, "pane.text"),
		    helpfile);

	XtPopup(HelpShell, XtGrabNone);
}

/* Function:	InitHelp() initializes the help subsystem.
 * Arguments:	parent:	Parent widget for "Help" button.
 * Returns:	Nothing.
 * Notes:
 */

void
InitHelp(parent)
Widget parent;
{
	Cardinal arg_cnt;
	Arg args[10];
	Widget w;

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, "Help"); arg_cnt++;
        w = XtCreateManagedWidget("help", commandWidgetClass, parent,
					args, arg_cnt);
	XtAddCallback(w, XtNcallback, Help, NULL);
}
