/* This file is part of xdir, an X-based directory browser.
 * It contains miscellaneous utility functions.
 *
 *	Created: 8/13/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com.
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/util.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_util_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/util.c,v 3.0 90/03/31 01:35:26 treese Exp $";
#endif

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <X11/Xos.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include "xdir.h"

/* Function:	SaveString() creates a copy of a string.
 * Arguments:	string:	String to save.
 * Returns:	A pointer to the new copy (char *).
 * Notes:
 */

char *
SaveString(string)
char *string;
{
	char *new;

	new = malloc(strlen(string) + 1);
	strcpy(new, string);
	return(new);
}

#ifdef ABORT_ON_XT_ERRORS
/* Function:	AbortOnXtError() aborts the program on an X error.
 * Arguments:	msg:	Toolkit error message.
 * Returns:	Nothing.
 * Notes:
 */

static void
AbortOnXtError(msg)
char *msg;
{
	fprintf(stderr, "%s\n", msg);
	abort();
}
#endif /* ABORT_ON_XT_ERRORS */

/* Function:	InitErrorHandlers() initializes error handling for xdir.
 * Arguments:	None.
 * Returns:	Nothing.
 * Notes:
 */

void
InitErrorHandlers()
{
#ifdef ABORT_ON_XT_ERRORS
	XtSetErrorHandler(AbortOnXtError); 
#endif /* ABORT_ON_XT_ERRORS */
}

/* Function:	IsDirectory() tests to see if a pathname is a directory.
 * Arguments:	path:	Pathname of file to test.
 * Returns:	True or False.
 * Notes:	False is returned if the directory is not accessible.
 */

Boolean
IsDirectory(path)
char *path;
{
	char fullpath[MAXPATHLEN];
	struct stat statbuf;

	if (path == NULL)
		return(False);
	MakeFullPath(path, fullpath);
	if (stat(fullpath, &statbuf)) {
		perror("xdir");
		return(False);
	}
	if (statbuf.st_mode & S_IFDIR) 
		return(True);
	else
		return(False);
}

/* Function:	MakeFullPath() creates the full pathname for the given file.
 * Arguments:	filename:	Name of the file in question.
 *		pathname:	Buffer for full name.
 * Returns:	Nothing.
 * Notes:
 */

void
MakeFullPath(filename, pathname)
char *filename;
char *pathname;
{
	strcpy(pathname, CurrentDirectory);
	strcat(pathname, "/");
	strcat(pathname, filename);
}

/* Function:	SetDirIconName() sets the icon name of the directory browser.
 * Arguments:	dir: Name to set with.
 * Returns:
 * Notes:
 *	This function creates an icon name of the form "Xdir: directory".
 */

void
SetDirIconName(dir)
char *dir;
{
	Arg args[10];
	Cardinal arg_cnt;
	char iconname[MAXPATHLEN];

	strcpy(iconname, "Xdir: ");
	strcat(iconname, dir);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNiconName, iconname); arg_cnt++;
	XtSetValues(TopWidget, args, arg_cnt);
}

/* Function:	GetWidgetHeight() gets the height of a widget.
 * Arguments:	w:	widget in question.
 * Returns:	The height (integer).
 * Notes:
 */

int
GetWidgetHeight(w)
Widget w;
{
    static int height;
    static Arg args[] = { {XtNheight, (XtArgVal)&height} } ;

    XtGetValues( w, args, (Cardinal)1 );
    return(height);
}

/* Function:	GetWidgetWidth() gets the width of a widget.
 * Arguments:	w:	Widget in question.
 * Returns:	The width (an integer).
 * Notes:
 */

int
GetWidgetWidth(w)
Widget w;
{
    static int width;
    static Arg args[] = { {XtNwidth, (XtArgVal)&width} } ;

    XtGetValues( w, args, (Cardinal)1 );
    return(width);
}

/* Function:	GetWidgetXLoc() gets the x location of a widget.
 * Arguments:	w:	Widget in question.
 * Returns:	The x coordinate (an integer).
 * Notes:
 */

int
GetWidgetXLoc(w)
Widget w;
{
    static int x;
    static Arg args[] = { {XtNx, (XtArgVal)&x} } ;

    XtGetValues(w, args, (Cardinal)1);
    return(x);
}

/* Function:	GetWidgetYLoc() gets the y location of a widget.
 * Arguments:	w:	Widget in question.
 * Returns:	The y coordinate (an integer).
 * Notes:
 */

int
GetWidgetYLoc(w)
Widget w;
{
    static int y;
    static Arg args[] = { {XtNy, (XtArgVal)&y} } ;

    XtGetValues(w, args, (Cardinal) 1);
    return(y);
}
