/* This file is part of xdir, an X-based directory browser.
 *
 *	Created: 8/13/88
 *
 *	Win Treese
 *	Cambridge Research Lab
 *	Digital Equipment Corporation
 *	treese@crl.dec.com
 *
 *	$Source: /trx/u2/treese/Src/Xdir.rel/RCS/xdir.c,v $
 *
 *          COPYRIGHT 1990
 *        DIGITAL EQUIPMENT CORPORATION
 *         MAYNARD, MASSACHUSETTS
 *        ALL RIGHTS RESERVED.
 *
 * THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND
 * SHOULD NOT BE CONSTRUED AS A COMMITMENT BY DIGITAL EQUIPMENT CORPORATION.
 * DIGITAL MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PURPOSE.  IT IS SUPPLIED "AS IS" WITHOUT EXPRESS OR IMPLIED
 * WARRANTY.
 *
 * IF THE SOFTWARE IS MODIFIED IN A MANNER CREATING DERIVATIVE COPYRIGHT
 * RIGHTS, APPROPRIATE LEGENDS MAY BE PLACED ON THE DERIVATIVE WORK IN
 * ADDITION TO THAT SET FORTH ABOVE.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Digital Equipment Corporation not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 */

#if !defined(lint) && !defined(SABER)
static char rcsid_xdir_c[] = "$Header: /trx/u2/treese/Src/Xdir.rel/RCS/xdir.c,v 3.0 90/03/31 01:35:29 treese Exp $";
#endif

#include <stdio.h>
#include <sys/param.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>

#define MAIN			/* Causes xdir.h to define global variables. */

#include "xdir.h"

/* Global variables. */

char CurrentSelectionName[MAXPATHLEN];
int CurrentSelectionIndex = 0;
char CurrentDirectory[MAXPATHLEN];
Widget TopWidget;
Widget MyListWidget;
Widget DirLabel;
Widget MyViewport;
XtAppContext XdirContext;
XdirResources resources;

/* Function:	FileSelected() is called when the user selects a file.
 *		If the same file is selected twice, then it is displayed
 *		appropriately (either by viewing the file or changing to
 *		the directory.
 * Arguments:	Standard Xt callback arguments.
 * Returns:	Nothing.
 * Notes:
 *	Viewing the item if selected twice is a hack until doubleclicks
 *	work.
 */

/*ARGSUSED*/
static void
FileSelected(w, client_data, ret_val)
Widget w;
caddr_t client_data;
caddr_t ret_val;
{
	XawListReturnStruct *ret_struct = (XawListReturnStruct *) ret_val;

	if (CurrentSelectionName[0] != EOS &&
	    streq(CurrentSelectionName, ret_struct->string)) {
		if (IsDirectory(CurrentSelectionName)) {
			DoChangeDir(CurrentSelectionName);
		}
		else
		  DisplayFile(CurrentSelectionName, NOEDIT);
	}
	else {
		strcpy(CurrentSelectionName, ret_struct->string);
		CurrentSelectionIndex = ret_struct->list_index;
	}
}

void
main(argc, argv)
int argc;
char *argv[];
{
	Widget pane;
	Arg args[10];
	Cardinal arg_cnt;
	char **file_list;

	TopWidget = XtAppInitialize(&XdirContext, "Xdir", NULL, ZERO,
				    &argc, argv, NULL, NULL, ZERO);
	InitErrorHandlers();
	InitResources(TopWidget);

	argc--; argv++;
#ifdef notdef
	while (argc >0 && *argv[0] == '-') {
		(*argv)++;
		while (**argv) switch (*(*argv)++) {
		}
	}
#endif
	if (*argv) {
		strcpy(CurrentDirectory, *argv);
	}
	else {
		if (getwd(CurrentDirectory) == NULL)
			ErrAbort("cannot find current directory!");
	}

	file_list = MakeFileList(CurrentDirectory);
	if (file_list == NULL)
		exit(1);

	arg_cnt = 0;
  	pane = XtCreateManagedWidget("pane", panedWidgetClass, TopWidget,
				      args, arg_cnt);

	InitMenus(pane);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlabel, CurrentDirectory); arg_cnt++;
	DirLabel = XtCreateManagedWidget("dirname", labelWidgetClass, 
				      pane, args, arg_cnt);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNallowVert, True); arg_cnt++;
	MyViewport = XtCreateManagedWidget("vport", viewportWidgetClass, 
				      pane, args, arg_cnt);

	arg_cnt = 0;
	XtSetArg(args[arg_cnt], XtNlist, file_list); arg_cnt++;
	MyListWidget = XtCreateManagedWidget("list", listWidgetClass,
					     MyViewport, args, arg_cnt);
	XtAddCallback(MyListWidget, XtNcallback, FileSelected,
		      (caddr_t) NULL);

	SetDirIconName(CurrentDirectory);

	InitButtonActions();
	InitTextDisplay();

	XtRealizeWidget(TopWidget);
	XtAppMainLoop(XdirContext);
}

